/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg;

import static org.fortiss.af3.exploration.alg.ExplorationAlgActivator.getDefault;
import static org.fortiss.af3.exploration.alg.service.ExplorationService.getService;

import org.eclipse.ui.IStartup;
import org.fortiss.af3.exploration.alg.contribution.Opt4JDseBackend;
import org.fortiss.af3.exploration.alg.service.IExplorationConstraintTransformationService;
import org.fortiss.af3.exploration.alg.service.IExplorationEvaluatationService;
import org.fortiss.af3.exploration.alg.service.IExplorationRepairService;
import org.fortiss.af3.exploration.alg.service.IExplorationService;
import org.fortiss.af3.exploration.alg.service.IExplorationTransformationService;
import org.fortiss.af3.exploration.alg.service.internal.ExplorationConstraintTransformationService;
import org.fortiss.af3.exploration.alg.service.internal.ExplorationEvaluationService;
import org.fortiss.af3.exploration.alg.service.internal.ExplorationRepairService;
import org.fortiss.af3.exploration.alg.service.internal.ExplorationTransformationService;
import org.fortiss.af3.exploration.backend.DseBackendHandler;
import org.osgi.framework.BundleContext;

/**
 * Centralized class that provides the startup routines of all services provided by this plugin.
 * 
 * The order of execution is as follows:
 * - Initialize own services.
 * - Register with the exploration plugin. (Make the "Parent" aware of me)
 * - Start own services.
 * 
 * @author diewald
 */
public final class ExplorationAlg implements IStartup {

	/** {@inheritDoc} */
	@Override
	public void earlyStartup() {
		initialize();
		registerBackend();
		start();
	}

	/**
	 * Initializes the {@link IExplorationService}s contained in this plugin (first service
	 * activation stage).
	 */
	public static void initialize() {
		registerServices();

		getService(IExplorationConstraintTransformationService.class).initializeService();
		getService(IExplorationEvaluatationService.class).initializeService();
		getService(IExplorationRepairService.class).initializeService();
		getService(IExplorationTransformationService.class).initializeService();
	}

	/** Registers the provided services with the {@link BundleContext}. */
	private static void registerServices() {
		BundleContext context = getDefault().getBundle().getBundleContext();

		context.registerService(IExplorationConstraintTransformationService.class.getName(),
				ExplorationConstraintTransformationService.getInstance(), null);
		context.registerService(IExplorationEvaluatationService.class.getName(),
				ExplorationEvaluationService.getInstance(), null);
		context.registerService(IExplorationRepairService.class.getName(),
				ExplorationRepairService.getInstance(), null);
		context.registerService(IExplorationTransformationService.class.getName(),
				ExplorationTransformationService.getInstance(), null);
	}

	/** Register the DSE backend with the exploration plugin. */
	private static void registerBackend() {
		DseBackendHandler.getInstance().registerDseBackend(new Opt4JDseBackend());
	}

	/**
	 * Starts the {@link IExplorationService}s contained in this plugin (second service activation
	 * stage).
	 */
	public static void start() {
		getService(IExplorationConstraintTransformationService.class).startService();
		// The Transformation Service must be started before the Evaluation Service such that it can
		// check for the availability of required transformation modules.
		getService(IExplorationTransformationService.class).startService();
		getService(IExplorationEvaluatationService.class).startService();
		getService(IExplorationRepairService.class).startService();
	}
}
