/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.partitionmapping;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.MappingEncoding.CREATE_STARTEGY;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.create.ComposableCreatorBase;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.FlatGenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.graph.DecoderDependencyGraph;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.random.RandomExploration;
import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;
import org.opt4j.core.problem.Creator;

/**
 * {@link Creator} for {@link PartitionMappingEncoding}s.
 * 
 * @author diewald
 */
public class PartitionMappingCreator extends ComposableCreatorBase<PartitionMappingEncoding> {

	/** Constructor. */
	public PartitionMappingCreator(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
			DecoderDependencyGraph depGraph) {
		super(systemModelAdapter, depGraph);
	}

	/** {@inheritDoc} */
	@Override
	public PartitionMappingEncoding createComposable(FlatGenotypeMap createdGenotypes,
			FlatPhenotypeMap<Phenotype> decodedPhenotypes) {
		// TODO In order to correctly choose the create strategy here, we'd need to know the
		// population size. For now, we assume an initial population size of 25.
		final int INIT_POP_SIZE = 25;
		// "Probability distribution" for the strategy selection: | 1 | 23 | 1 | ==> 25
		int strategySelIdx = RandomExploration.getInstance().nextInt(INIT_POP_SIZE);
		switch(strategySelIdx) {
			case 0:
				return new PartitionMappingEncoding(systemModelAdapter, CREATE_STARTEGY.MIN_SET);
			case 24:
				return new PartitionMappingEncoding(systemModelAdapter, CREATE_STARTEGY.MAX_SET);
			default:
				return new PartitionMappingEncoding(systemModelAdapter, CREATE_STARTEGY.RANDOM_SET);
		}
	}

	/** {@inheritDoc} */
	@Override
	public Class<? extends IExplorationEncoding> getCreatedEncodingType() {
		return PartitionMappingEncoding.class;
	}
}
