/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.ComposablePhenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.mapping.IMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.sysmodel.mapping.IMappingEntry;
import org.opt4j.core.Genotype;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.LinkedListMultimap;
import com.google.common.collect.ListMultimap;

/**
 * Marker interface for porting: Genotypes which allocate one request to a resource.
 * 
 * @author diewald
 */
public abstract class MappingEncoding<G extends Genotype, S extends IRequestAdapter<?>, T extends IResourceAdapter<?>, M extends IMappingEntry<S, T>>
		implements ComposablePhenotype<G>, IMappingEncoding<S, T, M>, Cloneable {

	/**
	 * Allocation map
	 * <ul>
	 * <li>key: The Requester (e.g., task)</li>
	 * <li>value: The target resource (e.g., cores)</li>
	 * <li>Multiple entry in the list represent redundant mapping</li>
	 */
	protected ListMultimap<S, M> allocationMap = LinkedListMultimap.create();

	public enum CREATE_STARTEGY {
		MIN_SET, RANDOM_SET, MAX_SET
	}

	/**
	 * Creates a copy of the internal allocation map of {@code this} {@link TaskMappingEncoding}.
	 */
	public ListMultimap<S, M> copyAllocationMap(Map<M, M> copyAssocMap) {
		ListMultimap<S, M> copiedMap = ArrayListMultimap.create();
		for(Entry<S, M> tmEntry : allocationMap.entries()) {
			@SuppressWarnings("unchecked") M mappingEntry = (M)tmEntry.getValue().clone();
			copiedMap.put(tmEntry.getKey(), mappingEntry);
			copyAssocMap.put(tmEntry.getValue(), mappingEntry);
		}
		return copiedMap;
	}

	/** {@inheritDoc} */
	@Override
	public synchronized Collection<T> getAllocatedResourcesFor(S requester) {
		Collection<T> allocatedResourcesForRequester = new ArrayList<>();
		for(IMappingEntry<S, T> currentMappingEntry : allocationMap.get(requester)) {
			allocatedResourcesForRequester.add(currentMappingEntry.getTarget());
		}
		return allocatedResourcesForRequester;
	}

	/** {@inheritDoc} */
	@Override
	public List<M> getMappingEntriesOf(S source) {
		return allocationMap.get(source);
	}

	/** {@inheritDoc} */
	@Override
	public boolean containsEntry(M entry) {
		return allocationMap.containsValue(entry);
	}

	/** {@inheritDoc} */
	@Override
	public Collection<M> getMappingEntries() {
		return allocationMap.values();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<S> getRequesters() {
		return allocationMap.keySet();
	}

	@Override
	public abstract MappingEncoding<G, S, T, M> clone();
}
