/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.mapping.IMappingEntry;

import com.google.common.collect.ClassToInstanceMap;
import com.google.common.collect.MutableClassToInstanceMap;

/**
 * 
 * @author diewald
 */
public abstract class MappingEntryBase<S extends IRequestAdapter<?>, T extends IResourceAdapter<?>>
		implements IMappingEntry<S, T>, Cloneable {
	/** Deployment target to which task is mapped */
	protected T target;

	/** {@link IDeployableComponentAdapter} that is "realized" by this {@link TaskMappingEntry}. */
	protected S source;

	/** Associated {@link IMappingEntry}s that were generated during the DSE process. */
	protected ClassToInstanceMap<IMappingEntry<S, T>> associatedEntries =
			MutableClassToInstanceMap.create();

	/** {@inheritDoc} */
	@Override
	public S getSource() {
		return source;
	}

	/** {@inheritDoc} */
	@Override
	public T getTarget() {
		return target;
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public void addAssociatedElement(IMappingEntry<S, T> assocEntry) {
		associatedEntries.put((Class<IMappingEntry<S, T>>)assocEntry.getClass(), assocEntry);
	}

	/** {@inheritDoc} */
	@Override
	public void rmAssociatedElement(IMappingEntry<S, T> entry) {
		associatedEntries.remove(entry.getClass());
	}

	/** {@inheritDoc} */
	@Override
	public Collection<IMappingEntry<S, T>> getAssociatedElement() {
		return associatedEntries.values();
	}

	/** {@inheritDoc} */
	@Override
	public <U extends IMappingEntry<S, T>> U getAssociatedElement(Class<U> type) {
		return associatedEntries.getInstance(type);
	}

	@Override
	public abstract IMappingEntry<S, T> clone();
}
