/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.comm.constraint;

import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.fortiss.af3.exploration.alg.dse.sysmodel.mapping.IMappingEntry;
import org.fortiss.af3.exploration.dsl_v2.model.expression.IExpression;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.model.ExplorationTarget;

/**
 * DSE-internal {@link ExplorationConstraint} that marks a group of {@link IMappingEntry}s that are
 * only allowed to communicate with each other or {@link IMappingEntry}s which ar enot part of any
 * other {@link InternalIsolatedCommunicationSet}.
 * 
 * @author diewald
 */
// TODO: Clarify the internal ExplorationConstraint concept to allow for a cleaner implementation.
@SuppressWarnings("javadoc")
public class InternalIsolatedCommunicationSet extends EObjectImpl
		implements ExplorationConstraint<Boolean>, Cloneable {

	private int id = 0;

	private String name = "";

	private String comment = "";

	private boolean isSoft = false;

	private Set<IMappingEntry> isolatedCommunicationSet;

	public InternalIsolatedCommunicationSet(Set<IMappingEntry> isolatedCommunicationSet) {
		this.isolatedCommunicationSet = isolatedCommunicationSet;
	}

	public InternalIsolatedCommunicationSet(InternalIsolatedCommunicationSet other) {
		this.isolatedCommunicationSet = new HashSet<>(other.isolatedCommunicationSet);
		this.name = other.name;
		this.comment = other.comment;
		this.isSoft = other.isSoft;
	}

	/** {@inheritDoc} */
	@Override
	public String getComment() {
		return comment;
	}

	/** {@inheritDoc} */
	@Override
	public void setComment(String value) {
		comment = value;
	}

	/** {@inheritDoc} */
	@Override
	public String getName() {
		return name;
	}

	/** {@inheritDoc} */
	@Override
	public void setName(String value) {
		name = value;
	}

	/** {@inheritDoc} */
	@Override
	public int getId() {
		return id;
	}

	/** {@inheritDoc} */
	@Override
	public void setId(int value) {
		id = value;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isIsSoft() {
		return isSoft;
	}

	/** {@inheritDoc} */
	@Override
	public void setIsSoft(boolean value) {
		isSoft = value;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isImplicit() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public void setImplicit(boolean value) {
		// NOP.
	}

	public Set<IMappingEntry> getIsolatedCommunicationSet() {
		return isolatedCommunicationSet;
	}

	public void setIsolatedCommunicationSet(Set<IMappingEntry> isolatedCommunicationSet) {
		this.isolatedCommunicationSet = isolatedCommunicationSet;
	}

	public void addToIsolatedCommunicationSet(Collection<? extends IMappingEntry> addEntries) {
		this.isolatedCommunicationSet.addAll(addEntries);
	}

	public void removeFromIsolatedCommunicationSet(Collection<? extends IMappingEntry> rmEntries) {
		this.isolatedCommunicationSet.removeAll(rmEntries);
	}

	public void replaceEntry(IMappingEntry oldEntry, IMappingEntry newEntry) {
		this.isolatedCommunicationSet.remove(oldEntry);
		this.isolatedCommunicationSet.add(newEntry);
	}

	public void replaceEntriesIfPresent(
			Map<? extends IMappingEntry, ? extends IMappingEntry> replacementMap) {
		for(Entry<? extends IMappingEntry, ? extends IMappingEntry> entry : replacementMap
				.entrySet()) {
			if(contains(entry.getKey())) {
				replaceEntry(entry.getKey(), entry.getValue());
			}
		}
	}

	public boolean contains(IMappingEntry entry) {
		return isolatedCommunicationSet.contains(entry);
	}

	@Override
	public Object clone() {
		return new InternalIsolatedCommunicationSet(this);
	}

	/** {@inheritDoc} */
	@Override
	public IExpression getExpression() {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public void setExpression(IExpression value) {
		// TODO Auto-generated method stub
	}

	/** {@inheritDoc} */
	@Override
	public EList<ExplorationTarget<Boolean>> getContainedTargets() {
		// TODO Auto-generated method stub
		return null;
	}
}
