/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.mating;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.opt4j.optimizers.ea.Mating;
import org.opt4j.optimizers.ea.MatingModule;

import com.google.inject.Provides;
import com.google.inject.Singleton;
import com.google.inject.name.Named;
import com.google.inject.name.Names;

/**
 * Guice module that binds the {@link MatingCrossoverMutateRepairModule} to Opt4J.
 * 
 * @author diewald
 */
public class MatingCrossoverMutateRepairModule extends MatingModule {

	/** References the input model adapter. */
	private final SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter;

	/** Constructor. */
	public MatingCrossoverMutateRepairModule(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		this.systemModelAdapter = systemModelAdapter;
	}

	/** {@inheritDoc} */
	@Override
	protected void config() {
		binder().bind(Mating.class).to(MatingCrossoverMutateRepair.class);

		binder().bind(SystemModelAdapter.class).annotatedWith(Names.named("SystemModelAdapter"))
				.toInstance(systemModelAdapter);
	}

	/** Returns the systemModelAdapter referenced by {@code this} module. */
	@Provides
	@Singleton
	@Named("SystemModelAdapter")
	public SystemModelAdapter<?, ?, ?, ?, ?, ?> getSystemModelAdapter() {
		return systemModelAdapter;
	}
}
