/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.partitionmapping;

import java.util.Collection;
import java.util.function.Consumer;
import java.util.function.Function;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.MutateBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;

/**
 * Base class for the mutation functions operating on the Partition to ExecutionUnit mapping, e.g.
 * cores.
 * 
 * @author diewald
 */
public abstract class MutatePartitionBase extends MutateBase<PartitionMappingEncoding> {
	/** Elements to which other elements may be allocated. */
	private Collection<IResourceAdapter<?>> allocationTargets;

	/** Defines the rate at which mutations shall happen: Typically 1 / size of the gene. */
	private double rate;

	/** Constructor. */
	public MutatePartitionBase(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		super(systemModelAdapter);
		// TODO: The statement below hard-wires the module.
		this.allocationTargets.addAll(systemModelAdapter.getDeploymentTargets());
		this.rate = 1 / allocationTargets.size();
	}

	/** Applies the given mutation {@link Function} to the allocation targets. */
	protected <R> void mutate(Consumer<IResourceAdapter<?>> manipulationFct) {
		for(IResourceAdapter<?> resource : allocationTargets) {
			if(rand.nextDouble() < rate) {
				manipulationFct.accept(resource);
			}
		}
	}
}
