/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.partitionmapping;

import java.util.function.Consumer;
import java.util.stream.Stream;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;

/**
 * Mutation operator that randomly selects resources from which to remove a random partition. The
 * rate is designed such that one resource is selected in average.
 * 
 * @author diewald
 */
public class MutatePartitionDecrement extends MutatePartitionBase {

	/** Constructor. */
	public MutatePartitionDecrement(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		super(systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public void mutate(PartitionMappingEncoding genotype, double p) {
		Consumer<IResourceAdapter<?>> remove = (res) -> genotype.removeRandomPartition(res);
		Stream<IResourceAdapter<?>> resStream = genotype.getResources().stream();
		resStream.filter((res) -> rand.nextDouble() < p).forEach(remove);
	}

}
