/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.safetyarch;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.MutateBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.opt4j.operators.mutate.Mutate;

/**
 * Base class for {@link Mutate} operators that manipulate the entries of a
 * {@link SafetyFunctionArchEncoding}.
 * 
 * @author diewald
 */
public abstract class MutateSafetyArchBase<C> extends MutateBase<SafetyFunctionArchEncoding<C>> {

	/** Constructor. */
	protected MutateSafetyArchBase(SystemModelAdapter<C, ?, ?, ?, ?, ?> systemModelAdapter) {
		super(systemModelAdapter);
	}

	/**
	 * Selects a collection of {@link SafetyFunctionArchEntry}s from the given
	 * {@link SafetyFunctionArchEncoding} whose element number depends on the given probability
	 * {@code p}.
	 */
	protected Collection<SafetyFunctionArchEntry<C>> selectSafetyFunctions(
			SafetyFunctionArchEncoding<C> genotype, double p) {
		Collection<SafetyFunctionArchEntry<C>> selEntries = new ArrayList<>();
		int size = genotype.size();
		int idx = rand.nextInt(size);
		for(int i = 0; i < genotype.size(); i++) {
			if(idx < p * size) {
				SafetyFunctionArchEntry<C> entry = genotype.getEntry(idx);
				if(entry != null) {
					selEntries.add(entry);
				}
			}
		}
		return selEntries;
	}
}
