/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskinstantiation;

import java.util.Collection;
import java.util.HashSet;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskmapping.MutateTasksBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.taskgraph.TaskGraph;
import org.opt4j.operators.mutate.Mutate;

/**
 * {@link Mutate} operator which manipulates the instantiation of the referenced
 * {@link IDeployableComponentAdapter} of a {@link InstantiatedTaskMappingEntry}. The instantiated
 * {@link IDeployableComponentAdapter} is randomly selected from the set of instantiatable
 * {@link IDeployableComponentAdapter}s of the associated {@link AbstractTaskMappingEntry}.
 * <p>
 * <b>NOTE:</b> Currently, this is a Nop operator, as a correct handling of {@link TaskGraph}s and
 * an appropriate reliability analysis must be worked out.
 * 
 * @author diewald
 */
public class MutateTaskInstance
		extends
		MutateTasksBase<InstantiatedTaskMappingEntry, InstantiatedTaskMappingEncoding<InstantiatedTaskMappingEntry>> {
	/**
	 * Collection of {@link IDeployableComponentAdapter}s which must be instantiated to obtain a
	 * "concrete" logical model".
	 */
	private Collection<IDeployableComponentAdapter<?>> abstractComponents;

	/** Constructor. */
	public MutateTaskInstance(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		super(systemModelAdapter);
		abstractComponents = new HashSet<IDeployableComponentAdapter<?>>();

		for(IDeployableComponentAdapter<?> deployableComponent : systemModelAdapter
				.getDeployableComponents()) {
			if(deployableComponent.isInstantiationRequired()) {
				abstractComponents.add(deployableComponent);
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public void mutate(InstantiatedTaskMappingEncoding<InstantiatedTaskMappingEntry> encoding,
			double rate) {
		if(rand.nextDouble() < rate) {
			// TODO: Evaluate mutation of the diverse implementation.
			@SuppressWarnings("unused") int applyToIdx = 0;
			if(abstractComponents.size() > 1) {
				applyToIdx = rand.nextInt(abstractComponents.size() - 1);
			}
			// candidates for this mutation operation.
		}
	}
}
