/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskmapping;

import java.util.Collection;

import org.conqat.lib.commons.collections.CollectionUtils;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.exception.ExplorationException;

/**
 * Mutate operator that decreases the redundancy of allocations.
 * 
 * @author huang
 */
public class MutateRedundancyIncrement<S extends TaskMappingEntry<IDeployableComponentAdapter<?>, Partition>, T extends TaskMappingEncoding<IDeployableComponentAdapter<?>, S>>
		extends MutateRedundancy<S, T> {
	/** Constructs a new increment redundancy mutation operator for a given exploration run */
	public MutateRedundancyIncrement(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		super(systemModelAdapter);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws ExplorationException
	 */
	@Override
	protected void mutate(T encoding, IDeployableComponentAdapter<?> deployableComponent,
			int minReplicatation, int maxReplication) throws ExplorationException {
		Collection<S> assignedResources = encoding.getMappingEntriesOf(deployableComponent);
		assert (assignedResources != null);

		// increase redundancy: the maximum replication should not be exceeded
		if(assignedResources.size() < maxReplication) {
			// This respects {@link ExcludeDeploymentConstraint}s
			// The following code ensures an equal distribution of temporal and spatial replication
			if(rand.nextBoolean()) {
				assignedResources.add(encoding.createRandomEntry(deployableComponent,
						CollectionUtils.asRandomAccessList(
								encoding.getAllocatedResourcesFor(deployableComponent))));
			} else {
				assignedResources.add(encoding.createRandomEntry(deployableComponent));
			}
		}
	}
}
