/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskmapping;

import org.fortiss.af3.exploration.alg.dse.TaskMappingFactory;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.MutateOperatorModuleBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.opt4j.operators.mutate.Mutate;

import com.google.inject.Module;

/**
 * This {@link Module} registers {@link Mutate} operators in Opt4J which manipulate the
 * {@link TaskMappingEncoding}s.
 * 
 * @author huang
 */
public class MutateTaskMappingModule extends MutateOperatorModuleBase {
	/** Representation of input system */
	private SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter;

	/** Constructor where the input system model has to be specified. */
	public MutateTaskMappingModule(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		this.systemModelAdapter = systemModelAdapter;
	}

	/** {@inheritDoc} */
	@Override
	public void config() {
		super.config();
		TaskMappingFactory<? extends TaskMappingEntry<?, ?>, ? extends TaskMappingEncoding<?, ?>> tmFactory =
				TaskMappingFactory.getInstance();

		addOperatorInstance(AbstractTaskMappingEncoding.class,
				tmFactory.createMutateAllocationOperator(systemModelAdapter));
		addOperatorInstance(AbstractTaskMappingEncoding.class,
				tmFactory.createMutateRedundancyDecrementOperator(systemModelAdapter));
		addOperatorInstance(AbstractTaskMappingEncoding.class,
				tmFactory.createMutateRedundancyIncrementOperator(systemModelAdapter));
	}
}
