/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem;

import static org.fortiss.af3.exploration.alg.util.ExplorationAlgUtils.rethrow;

import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.DSEFactory;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.ArchitectureExplorationEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.graph.DecoderDependencyGraph;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.ArchitectureSolution;
import org.fortiss.af3.exploration.alg.dse.evaluator.ArchitectureEvaluator;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;
import org.fortiss.af3.exploration.moea.model.DseSpecification;

/**
 * Problem module for fail-silent execution model, that encapsulates a creator, a decoder and
 * the set of goal-specific evaluators.
 * 
 * @author huang
 */
public class ArchitectureExplorationProblemModule
		extends
		DseProblemModuleBase<ArchitectureExplorationEncoding, ArchitectureSolution, ArchitectureDecoder> {

	/** Constructor. */
	public ArchitectureExplorationProblemModule(DseSpecification dse,
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
			Set<Class<? extends IExplorationEncoding>> requestedSolutions) {
		super(dse, systemModelAdapter, requestedSolutions);
	}

	/** Configure genotype creator (execution model dependent), problem decoder and goal evaluators. */
	@Override
	protected void config() {
		/*
		 * Bind the Creators and the Decoders to the opt4j optimization problem. The evaluators are
		 * added separately.
		 */
		DSEFactory dseFactory = DSEFactory.getInstance();

		ArchitectureEvaluator archEvaluator =
				dseFactory.createArchitectureEvaluator(systemModelAdapter);

		// Dependency graph for the execution order.
		DecoderDependencyGraph execDepGraph =
				new DecoderDependencyGraph(systemModelAdapter, requestedSolutions);

		// TODO: construct the execution dependency graph elsewhere?
		bindProblem(dseFactory.createArchitectureExplorationCreator(dse, systemModelAdapter),
				dseFactory.createArchitectureDecoder(systemModelAdapter, execDepGraph),
				archEvaluator);

		// Create/Bind the subproblems of the architectural exploration problem.
		try {
			dseFactory.createSubProblems(this, dse, systemModelAdapter, execDepGraph);
		} catch(Exception e) {
			throw new RuntimeException("Failed setting up the DSE Problem instance.", e);
		}

		// Create evaluators wrappers (based on provided goal specification)
		try {
			addGoalEvaluators(archEvaluator);
		} catch(Exception e) {
			throw rethrow(e);
		}
	}
}
