/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator;

import java.util.Collection;
import java.util.Iterator;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate.CompositeEvaluator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate.CompositeEvaluatorBase;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.ArchitectureSolution;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.opt4j.core.Value;

import com.google.inject.Inject;

/**
 * {@link CompositeEvaluator} of an architectural exploration problem. This implementation sums the
 * results of an evaluation, if it is defined for multiple sub problems, in the merge method.
 * 
 * @author diewald
 */
public class ArchitectureEvaluator extends CompositeEvaluatorBase<ArchitectureSolution> implements
		ITopLevelEvaluator {

	/** Default constructor which requires a {@link SystemModelAdapter} to access the input models. */
	@Inject
	public ArchitectureEvaluator(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		super(systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public Value<?> mergeSubEvaluations(Collection<Value<?>> results) {
		Value<?> evalValue = null;
		Iterator<Value<?>> resultIterator = results.iterator();
		while(resultIterator.hasNext()) {
			Value<?> subResult = resultIterator.next();
			// TODO: Clarify the semantics of the aggregation of results. Some sort of add operation
			// will be need for the "Value" type. The 'getDouble' method is not available in
			// general.
			evalValue = subResult;
			// evalValue += subResults.getDouble();
		}

		return evalValue;
	}
}
