/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.MappingEncoding;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.dsl_v2.model.booleanp.Implies;
import org.fortiss.af3.exploration.dsl_v2.model.booleanp.allocation.Allocation;
import org.fortiss.af3.exploration.dsl_v2.model.booleanp.allocation.Dislocation;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.fortiss.tooling.base.model.element.IModelElement;

/**
 * An expression evaluator which is able to handle the location operators and the internal mapping
 * representation.
 * 
 * @author diewald
 */
public abstract class MappingEvaluator
		extends ExpressionEvaluatorBase<MappingEncoding, Double, ExplorationTarget<Double>> {

	/** Reference to the mapping which shall be evaluated here. */
	protected MappingEncoding mapping;

	/** Empty prototype constructor. */
	public MappingEvaluator() {
		super();
	}

	/** Constructor. */
	public MappingEvaluator(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
			ExplorationTarget<Double> explorationTarget) throws Exception {
		super(systemModelAdapter, explorationTarget);
	}

	/** {@inheritDoc} */
	@Override
	public Object visit(Allocation alloc) throws Exception {
		// TODO: Here, we could use the requester / resource framework for the return types. In
		// addition we would obtain a generic mapping evaluator.
		IModelElement requester = alloc.getLeft().accept(this, IModelElement.class);
		IModelElement resource = alloc.getRight().accept(this, IModelElement.class);

		IRequestAdapter<?> requestAdp = systemModelAdapter.getDeployableComponentOf(requester);

		Collection<IResourceAdapter<?>> allocResourceAdps =
				mapping.getAllocatedResourcesFor(requestAdp);
		for(IResourceAdapter<?> allocResource : allocResourceAdps) {
			if(allocResource.getObject() == resource) {
				return Boolean.valueOf(true);
			}
		}

		return Boolean.valueOf(false);
	}

	/** {@inheritDoc} */
	@Override
	public Object visit(Dislocation disLoc) throws Exception {
		IModelElement requester = disLoc.getLeft().accept(this, IModelElement.class);
		IModelElement resource = disLoc.getRight().accept(this, IModelElement.class);

		IRequestAdapter<?> requestAdp = systemModelAdapter.getDeployableComponentOf(requester);

		Collection<IResourceAdapter<?>> allocResourceAdps =
				mapping.getAllocatedResourcesFor(requestAdp);
		for(IResourceAdapter<?> allocResource : allocResourceAdps) {
			if(allocResource.getObject() == resource) {
				return Boolean.valueOf(false);
			}
		}

		return Boolean.valueOf(true);
	}

	/** {@inheritDoc} */
	@Override
	public Object visit(Implies implies) throws Exception {
		// FIXME: How to implement implies?
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public boolean validateInputs() throws Exception {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredInputTypes() {
		Collection<Class<?>> requiredTypes = new ArrayList<>();
		requiredTypes.add(MappingEncoding.class);
		return requiredTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Class<?> getPrimaryInputType() {
		return MappingEncoding.class;
	}
}
