/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.modeltransformation.element;

import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.modeltransformation.ITransformationModule;
import org.fortiss.af3.exploration.alg.exception.TransformationModuleException;
import org.fortiss.af3.exploration.alg.graph.IDependencyModule;
import org.fortiss.af3.exploration.alg.service.ExplorationEncodingMap;
import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;

/**
 * Interface for operations that are common to all Element Generation/Transformation Modules.
 * 
 * @author diewald
 */
public interface IElementTransformationModule<T extends EObject, O extends EObject>
		extends ITransformationModule<T>, IDependencyModule<T, T> {

	/** Returns the Class (type) on which a concrete module operates. */
	Class<T> getElementClass();

	/**
	 * Returns the Class (type) that contains the elements on which {@code this}
	 * {@link IElementTransformationModule} operates.
	 */
	Class<O> getElementContainerType();

	/**
	 * Performs a model transformation for the given model element. For the transofrmation, the
	 * additional inputs can be used.
	 * 
	 * @param inputElement
	 *            Model element to be transformed.
	 * @param baseModels
	 *            Set of (original) base models.
	 * @param explorationEncodings
	 *            Encodings calculated by the DSE.
	 * @param transformedModels
	 *            Set of transformed/already manipulated models.
	 * @param outputModel
	 *            Equivalent to the returned model.
	 * @return Identical to {@code outputModel}.
	 * @throws TransformationModuleException
	 *             If any operation during the transformation of the given model element fails.
	 */
	// TODO: update method doc.
	// TODO: use the generic 'T' to specify the parameter 'inputElement'. EObject has been chosen
	// for now to avoid a compile error in ElementTransformtationStrategy for now but this
	// workaround is not type safe.
	<N extends EObject, S extends EObject> O transformElement(EObject inputElement,
			Map<Class<? extends S>, S> baseModels,
			ExplorationEncodingMap<IExplorationEncoding> explorationEncodings,
			Map<Class<? extends N>, N> transformedModels, O outputModel)
			throws TransformationModuleException;
}
