/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel;

import org.fortiss.af3.exploration.alg.dse.TaskMappingFactory;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.taskmapping.FaultDetectionVotingTaskMappingCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.taskmapping.TaskMappingCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FaultDetectionVotingTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FaultDetectionVotingTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.graph.DecoderDependencyGraph;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.crossover.CrossoverRandManyAllocation;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.crossover.CrossoverSingleAllocation;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskmapping.MutateAllocation;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskmapping.MutateRedundancyDecrement;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskmapping.MutateRedundancyIncrement;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.EvaluatorWrapper;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.StrictTTDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.comm.MessageDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.instantiatemapping.AbstractTaskMappingDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.instantiatemapping.FaultDetectionVotingAbstractTaskMappingDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.instantiatetaskgraph.InstantiatedTaskMappingDecoderAcyclic;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.StrictTTSchedule;
import org.fortiss.af3.exploration.alg.dse.evaluator.constraint.ComponentMultiAllocationConstraintEvaluator;
import org.fortiss.af3.exploration.alg.dse.evaluator.constraint.ComponentMultiDislocationConstraintEvaluator;
import org.fortiss.af3.exploration.alg.dse.evaluator.constraint.DeadlineConstraintEvaluator;
import org.fortiss.af3.exploration.alg.dse.evaluator.constraint.PeriodConstraintEvaluator;
import org.fortiss.af3.exploration.alg.dse.evaluator.constraint.SafetyIntegrityLevelConstraintEvaluator;
import org.fortiss.af3.exploration.alg.dse.evaluator.objective.FaultDetectionVotingReliabilityEvaluator;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.sched.proc.EDFSchedulerBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.sched.proc.FaultDetectionVotingEDFScheduler;
import org.fortiss.af3.exploration.model.ExplorationSpecification;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.fortiss.af3.exploration.model.time.DeadlineConstraint;
import org.fortiss.af3.exploration.model.time.PeriodConstraint;
import org.fortiss.af3.exploration.moea.model.DseSpecification;
import org.fortiss.af3.exploration.moea.model.predefined.ComponentMultiAllocationConstraint;
import org.fortiss.af3.exploration.moea.model.predefined.ComponentMultiDislocationConstraint;
import org.fortiss.af3.exploration.moea.model.predefined.FailureMinObjective;
import org.fortiss.af3.exploration.moea.model.predefined.SafetyIntegrityLevelConstraint;
import org.fortiss.af3.safety.model.SIL;

import com.google.inject.TypeLiteral;

/**
 * This class implements the creation of the EavaluatorWrapper for the fault detection by voting
 * model.
 * 
 * @author diewald
 */
public class FaultDetectionVotingExecModelFactory extends
		TaskMappingFactory<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding> {

	/** The instance (singleton) of the EvaluatorWrapper factory. */
	public static TaskMappingFactory<?, ?> getInstance() {
		if(instance == null) {
			instance = new FailSilentExecModelFactory();
		}
		return instance;
	}

	/* Construct Creators for the task mapping. */

	/** {@inheritDoc} */
	@Override
	public TypeLiteral<FaultDetectionVotingTaskMappingEncoding> createGenotypeTypeLiteral() {
		return new TypeLiteral<FaultDetectionVotingTaskMappingEncoding>() {
			// Anonymous.
		};
	}

	/** Creates a TaskMappingCreator that matches the given execution model. */
	@Override
	public TaskMappingCreator<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>
			createTaskMappingCreator(DseSpecification dse,
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
					DecoderDependencyGraph execDepGraph) {
		return new FaultDetectionVotingTaskMappingCreator(dse, systemModelAdapter, execDepGraph);
	}

	/* Construct execution model specific operators. */

	/** {@inheritDoc} */
	@Override
	public MutateAllocation<?, ?> createMutateAllocationOperator(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		return new MutateAllocation<AbstractTaskMappingEntry, AbstractTaskMappingEncoding>(
				systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public MutateRedundancyDecrement<?, ?> createMutateRedundancyDecrementOperator(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		return new MutateRedundancyDecrement<AbstractTaskMappingEntry, AbstractTaskMappingEncoding>(
				systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public MutateRedundancyIncrement<?, ?> createMutateRedundancyIncrementOperator(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		return new MutateRedundancyIncrement<AbstractTaskMappingEntry, AbstractTaskMappingEncoding>(
				systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public CrossoverRandManyAllocation<?, ?> createCrossoverRandManyAllocationOperator(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter, double rate) {
		return new CrossoverRandManyAllocation<AbstractTaskMappingEntry, AbstractTaskMappingEncoding>(
				systemModelAdapter, rate);
	}

	/** {@inheritDoc} */
	@Override
	public CrossoverSingleAllocation<?, ?> createCrossoverSingleAllocationOperator(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter, double rate) {
		return new CrossoverSingleAllocation<AbstractTaskMappingEntry, AbstractTaskMappingEncoding>(
				systemModelAdapter, rate);
	}

	/*
	 * Construct Decoders and Schedules for the genotypes, aka task mappings.
	 */

	/** {@inheritDoc} */
	@Override
	public AbstractTaskMappingDecoder<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>
			createAbstractTaskMappingDecoder(
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
					DseSpecification explorationDefinition) {
		return new FaultDetectionVotingAbstractTaskMappingDecoder(systemModelAdapter,
				explorationDefinition);
	}

	/** {@inheritDoc} */
	@SuppressWarnings({"rawtypes", "unchecked"})
	@Override
	// TODO: identify why a warning is generated here.
	public InstantiatedTaskMappingDecoderAcyclic<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>
			createAbstractToAcyclicTaskGraphDecoder(
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		return new InstantiatedTaskMappingDecoderAcyclic(systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public MessageDecoder<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>
			createMessageDecoder(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		return new MessageDecoder<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>(
				systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public StrictTTDecoder<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>
			createStrictTTDecoder(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
					DseSpecification explorationDefinition) {
		return new StrictTTDecoder<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>(
				systemModelAdapter, explorationDefinition);
	}

	/** {@inheritDoc} */
	@Override
	public EDFSchedulerBase<?, ?> createEDFScheduler(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
			ExplorationSpecification expSpec) {
		return new FaultDetectionVotingEDFScheduler(systemModelAdapter, expSpec);
	}

	/* Construct Evaluators for Optimization Goals. */

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public <RT, ET extends ExplorationTarget<RT>> EvaluatorWrapper<?, Double, FailureMinObjective>
			createFailureMinObjectiveWrapper(ExplorationTarget<?> goal, int priority,
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) throws Exception {
		return new EvaluatorWrapper<StrictTTSchedule<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>, Double, FailureMinObjective>(
				(FailureMinObjective)goal,
				new FaultDetectionVotingReliabilityEvaluator(systemModelAdapter,
						(FailureMinObjective)goal),
				priority,
				(Class<? extends StrictTTSchedule<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>>)StrictTTSchedule.class);
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public <RT, ET extends ExplorationTarget<RT>> EvaluatorWrapper<?, Double, DeadlineConstraint>
			createDeadlineConstraintWrapper(ExplorationTarget<?> goal, int priority,
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) throws Exception {
		return new EvaluatorWrapper<StrictTTSchedule<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>, Double, DeadlineConstraint>(
				(DeadlineConstraint)goal,
				new DeadlineConstraintEvaluator<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>(
						systemModelAdapter, (DeadlineConstraint)goal),
				priority,
				(Class<? extends StrictTTSchedule<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>>)StrictTTSchedule.class);
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public <RT, ET extends ExplorationTarget<RT>> EvaluatorWrapper<?, Double, PeriodConstraint>
			createPeriodConstraintWrapper(ExplorationTarget<?> goal, int priority,
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) throws Exception {
		return new EvaluatorWrapper<StrictTTSchedule<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>, Double, PeriodConstraint>(
				(PeriodConstraint)goal,
				new PeriodConstraintEvaluator<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>(
						systemModelAdapter, (PeriodConstraint)goal),
				priority,
				(Class<? extends StrictTTSchedule<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>>)StrictTTSchedule.class);
	}

	/** {@inheritDoc} */
	// TODO: adjust when the constraint is defined for mappings.
	@Override
	public <RT, ET extends ExplorationTarget<RT>>
			EvaluatorWrapper<?, Boolean, ComponentMultiAllocationConstraint>
			createComponentMultiAllocationConstraintWrapper(ExplorationTarget<?> goal, int priority,
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) throws Exception {
		return new EvaluatorWrapper<FaultDetectionVotingTaskMappingEncoding, Boolean, ComponentMultiAllocationConstraint>(
				(ComponentMultiAllocationConstraint)goal,
				new ComponentMultiAllocationConstraintEvaluator<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>(
						systemModelAdapter, (ComponentMultiAllocationConstraint)goal),
				priority, FaultDetectionVotingTaskMappingEncoding.class);
	}

	/** {@inheritDoc} */
	// TODO: adjust when the constraint is defined for mappings.
	@Override
	public <RT, ET extends ExplorationTarget<RT>>
			EvaluatorWrapper<?, Boolean, ComponentMultiDislocationConstraint>
			createComponentMultiDislocationConstraintWrapper(ExplorationTarget<?> goal,
					int priority, SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter)
					throws Exception {
		return new EvaluatorWrapper<FaultDetectionVotingTaskMappingEncoding, Boolean, ComponentMultiDislocationConstraint>(
				(ComponentMultiDislocationConstraint)goal,
				new ComponentMultiDislocationConstraintEvaluator<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>(
						systemModelAdapter, (ComponentMultiDislocationConstraint)goal),
				priority, FaultDetectionVotingTaskMappingEncoding.class);
	}

	/** {@inheritDoc} */
	@Override
	public <RT, ET extends ExplorationTarget<RT>>
			EvaluatorWrapper<?, SIL, SafetyIntegrityLevelConstraint>
			createSafetyIntegrityLevelConstraintWrapper(ExplorationTarget<?> goal, int priority,
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) throws Exception {
		return new EvaluatorWrapper<FaultDetectionVotingTaskMappingEncoding, SIL, SafetyIntegrityLevelConstraint>(
				(SafetyIntegrityLevelConstraint)goal,
				new SafetyIntegrityLevelConstraintEvaluator<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding>(
						systemModelAdapter, (SafetyIntegrityLevelConstraint)goal),
				priority, FaultDetectionVotingTaskMappingEncoding.class);
	}
}
