/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3;

import static org.fortiss.tooling.kernel.utils.EcoreUtils.pickFirstInstanceOf;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.safety.model.annotation.SafeFailureFraction;

/**
 * Adapter between AF3 {@link ExecutionUnit} (=deployment target of
 * {@link IDeployableComponentAdapter}s) and the DSE.
 * 
 * @author barner
 */
public class AF3ExecutionUnitAdapter extends AF3ResourceAdapter<ExecutionUnit> implements
		IExecutionUnitAdapter<ExecutionUnit> {
	/** Execution unit safe failure fraction annotation. */
	private SafeFailureFraction safeFailureFraction;

	/**
	 * Creates a {@link IExecutionUnitAdapter} for an {@link ExecutionUnit}.
	 * 
	 * @param executionUnit
	 *            execution unit model element.
	 * @param failureProb
	 *            failure probability of the given {@link ExecutionUnit}.
	 * @throws Exception
	 */
	public AF3ExecutionUnitAdapter(ExecutionUnit executionUnit, Double failureProb)
			throws Exception {
		super(executionUnit, failureProb);

		safeFailureFraction =
				pickFirstInstanceOf(SafeFailureFraction.class, executionUnit.getSpecifications());
	}

	/** {@inheritDoc} */
	@Override
	public double getSafeFailureFraction() {
		return safeFailureFraction.getValue();
	}
}
