/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.platform.model.IVirtualizationPlatformArchitectureElement;
import org.fortiss.tooling.base.model.element.IModelElement;

/**
 * Base adapter for concrete system model elements derived from {@link IResourceAdapter}.
 * 
 * @author barner
 */
// TODO: Use extends IResource, here? Same would apply to Requester/resource.
public class AF3ResourceAdapter<T extends IModelElement> extends AF3ModelElementAdapter<T>
		implements IResourceAdapter<T> {
	/** Failure rate annotation */
	private Double failureRate;

	/**
	 * Creates a {@link IExecutionUnitAdapter} for an {@link ExecutionUnit}.
	 */
	public AF3ResourceAdapter(T modelElement, Double failureRate) throws Exception {
		super(modelElement);

		this.failureRate = failureRate;
	}

	/** {@inheritDoc} */
	@Override
	public Double getFailureRate() {
		return failureRate;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isVirtual() {
		return getObject() instanceof IVirtualizationPlatformArchitectureElement;
	}
}
