/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3;

import java.util.Collection;
import java.util.Map;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceConnectionAdapter;
import org.fortiss.tooling.base.model.element.IConnection;

/**
 * Adapter between the AF3 {@link IConnection} and the DSE.
 * 
 * @author diewald
 */
public class AF3ResourceConnectionAdapter implements IResourceConnectionAdapter {

	/** The (actual) source element of an AF3 connection in the platform. */
	IResourceAdapter<?> sourceResource;

	/** The (actual) target element of an AF3 connection in the platform. */
	Collection<IResourceAdapter<?>> targetResources;

	/** Map which contains information about the direction of the source to each target. */
	Map<IResourceAdapter<?>, ConnectionType> targetDirectionMap;

	/** Constructor. */
	public AF3ResourceConnectionAdapter(IResourceAdapter<?> source,
			Collection<IResourceAdapter<?>> targetResourceList) {
		this.sourceResource = source;
		this.targetResources = targetResourceList;
	}

	/**
	 * Default constructor that allows to pass a map which contains information about the direction
	 * of the source to each target.
	 */
	public AF3ResourceConnectionAdapter(IResourceAdapter<?> source,
			Map<IResourceAdapter<?>, ConnectionType> targetResourceList) {
		this(source, targetResourceList.keySet());
		this.targetDirectionMap = targetResourceList;
	}

	/** Compatibility method for non-multicast AF3 versions. */
	public void addTargetResource(IResourceAdapter<?> targetResource) {
		targetResources.add(targetResource);
	}

	/** {@inheritDoc} */
	@Override
	public IResourceAdapter<?> getSourceResource() {
		return sourceResource;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<IResourceAdapter<?>> getTargetResources() {
		return targetResources;
	}

	/** {@inheritDoc} */
	@Override
	public ConnectionType getConnectionType(IResourceAdapter<?> targetResource) {
		if(targetDirectionMap.get(targetResource) != null) {
			return targetDirectionMap.get(targetResource);
		}
		return ConnectionType.BIDIRECTIONAL;
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		String sourceName = sourceResource.getName();
		String targetNames = "";
		for(IResourceAdapter<?> target : targetResources) {
			targetNames += target.getName() + ", ";
		}
		return sourceName + "->[" + targetNames + "]";
	}
}
