/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.graph;

import java.util.Collection;

/**
 * Interface to be implemented by operations that have dependencies and which are used in a process
 * like the DSE.
 * 
 * @author diewald
 */
// TODO: the Generic "I" is intended to define the input types. Unfortunately, using it in the
// transformation framework, causes some issues. In a future version, check the possibilities to use
// this parameter. Also apply this modification in the DependencyGraph class and its users in the
// Decoders and Element-wise transformation framework.
public interface IDependencyModule<I, O> {
	/**
	 * Returns the collection of input types that are required as direct
	 * inputs to {@code this} operation.
	 */
	Collection<Class<?>> getRequiredTypes();

	/**
	 * Returns the collection of input types that may provide optional information
	 * for the operation applied to the required inputs but which are not required for the
	 * operation.
	 */
	Collection<Class<?>> getOptionalTypes();

	/** Returns the resulting type of {@code this} operation. */
	Class<? super O> getOutputType();
}
