/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.port.plot;

import javax.swing.JFrame;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.StrictTTSchedule;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CategoryPlot;

/**
 * {@link SchedulePlotter} is responsible for visualizing the schedule that is generated for a
 * single Pareto-optimal solution. It illustrates both the execution unit schedules and the bus
 * schedules.
 * 
 * // TODO: generalize to common schedule class
 * 
 * @author diewald
 */
public class SchedulePlotter extends JFrame {

	/** Holds the schedule that has been calculated for the selected solution. */
	private StrictTTSchedule<?, ?> schedule;

	/**
	 * Constructor.
	 * 
	 * @param schedule
	 *            The scheduler provided by the solution
	 */
	public SchedulePlotter(StrictTTSchedule<?, ?> schedule) {
		super();
		this.schedule = schedule;
	}

	/** Draws the Gantt plot displaying the schedule. */
	public void createPlot() {

		/* Retrieve the tasks in a form convenient for plotting. */
		TaskSeriesCollectionFromSchedule dataset = new TaskSeriesCollectionFromSchedule(schedule);

		/* Initialize the chart. */
		JFreeChart chart = ChartFactory.createGanttChart("Time Triggered Execution Schedule", // chart
																								// title
				"Cores", // domain axis label
				"Time", // range axis label
				dataset, // data
				false, // include legend
				true, // tooltips
				false // urls
		);
		CategoryPlot plot = (CategoryPlot)chart.getPlot();
		plot.setRangeAxis(new NumberAxis("Time"));

		/* Assign the time slot specific renderer. */
		ScheduleGanttRenderer renderer = new ScheduleGanttRenderer(schedule);
		plot.setRenderer(renderer);

		/* Actually paint the window with the given dimensions. */
		ChartPanel chartPanel = new ChartPanel(chart);
		chartPanel.setPreferredSize(new java.awt.Dimension(500, 270));
		chartPanel.addChartMouseListener(new MessagePathOnClicked(plot, chartPanel, schedule));
		setContentPane(chartPanel);
		pack();
		setVisible(true);
	}
}
