/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.port.plot;

import java.util.HashMap;
import java.util.Map;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.StrictTTSchedule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.StrictTTSchedule.ResourceSchedule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.TimeSlot;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ICommunicationResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.comm.Message;
import org.fortiss.af3.exploration.alg.dse.sysmodel.mapping.IMappingEntry;
import org.jfree.chart.labels.StandardCategoryToolTipGenerator;
import org.jfree.data.category.CategoryDataset;

/**
 * Generates a tooltip label displaying information about the {@link TimeSlot} when the user
 * places the mouse pointer over it.
 * 
 * @author diewald
 */
public class ScheduleToolTipGenerator extends StandardCategoryToolTipGenerator {

	/** {@link StrictTTSchedule} for which the labels are generated. */
	private StrictTTSchedule<?, ?> schedule;

	/** holds row from previous call */
	private int rowLabel;
	/** holds column from previous call */
	private int colLabel;

	/** Previous {@link TimeSlot} for which a tooltip was generated. */
	Map<IResourceAdapter<?>, TimeSlot> prevSlot;

	/** Constructor. */
	public ScheduleToolTipGenerator(StrictTTSchedule<?, ?> schedule) {
		super();
		this.schedule = schedule;
		prevSlot = new HashMap<IResourceAdapter<?>, TimeSlot>();
	}

	/** {@inheritDoc} */
	@Override
	public String generateToolTip(CategoryDataset dataset, int row, int column) {
		if(rowLabel != row || colLabel != column) {
			rowLabel = row;
			colLabel = column;
			prevSlot.clear();
		}

		String taskAndResourceName = null;

		Comparable<?> rkey = dataset.getRowKey(row);
		Comparable<?> ckey = dataset.getColumnKey(column);

		if(rkey.equals("processing")) {
			IResourceAdapter<?> resource = null;
			for(IResourceAdapter<?> currentResource : schedule.getResources()) {
				if(currentResource.getName().equals(ckey)) {
					resource = currentResource;
					break;
				}
			}
			if(resource != null) {
				ResourceSchedule resourceSchedule = schedule.getResourceSchedule(resource);

				TimeSlot currentSlot = prevSlot.get(resource);
				if(currentSlot == null) {
					currentSlot = resourceSchedule.firstKey();
				} else {
					currentSlot = resourceSchedule.higherKey(currentSlot);
				}

				if(resource instanceof IExecutionUnitAdapter) {
					if(currentSlot != null) {
						IMappingEntry<IDeployableComponentAdapter<?>, ?> mappingEntry =
								currentSlot.getDeployedRequester();
						if(mappingEntry != null) {
							IDeployableComponentAdapter<?> component = mappingEntry.getSource();
							if(component != null) {
								prevSlot.put(resource, currentSlot);
								taskAndResourceName =
										component.getName() + " -> " + resource.getName();
							}
						}
					}
				} else if(resource instanceof ICommunicationResourceAdapter) {
					if(currentSlot != null) {
						Message message = currentSlot.getDeployedMessage();
						if(message != null) {
							return message.toString();
						}
					}
				}
			}
		}
		return taskAndResourceName;
	}
}
