/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.exception.ExplorationException;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.model.GenericConstraint;

/**
 * This service transforms a collection of given {@link GenericConstraint}s to
 * {@link ExplorationConstraint}s that can be processed internally by the DSE, e.g. in operators,
 * such that the DSE's efficiency can be increased. It does so by providing registered
 * {@link IExplorationContraintTransformationModule}s with the {@link GenericConstraint}s for which
 * the modules were registered and collecting the transformation results of each module.
 * 
 * @author diewald
 */
public interface IExplorationConstraintTransformationService extends
		IExplorationService<IExplorationContraintTransformationModule<?>> {

	// TODO: Should we use a separate marker for external Constrains?
	/**
	 * Returns the collection of transformed {@link ExplorationConstraint}s produced by the
	 * registered {@link IExplorationContraintTransformationModule}s.
	 * 
	 * @param systemModelAdapter
	 *            Abstraction of the input models.
	 * @param genericConstraints
	 *            Collection of constraints possibly pointing to external constraint models.
	 * @return Collection of transformed {@link ExplorationConstraint}s.
	 * @throws ExplorationException
	 *             if any of the registered transformation fails.
	 */
	Collection<ExplorationConstraint<?>> getTransformedConstraints(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
			Collection<GenericConstraint<?>> genericConstraints) throws ExplorationException;
}
