/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.exception.ExplorationException;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.model.GenericConstraint;
import org.fortiss.tooling.kernel.service.base.IEObjectAware;

/**
 * Module that transforms constraints of an external constraint model into
 * {@link ExplorationConstraint}s that are directly supported by the DSE.
 * 
 * @author diewald
 */
public interface IExplorationContraintTransformationModule<T extends GenericConstraint<?>> extends
		IEObjectAware<T>, IExplorationModule {

	/**
	 * Transforms constraints encoded in the given {@link GenericConstraint} to
	 * {@link ExplorationConstraint}s that are directly supported by the DSE. The resulting
	 * constraints may be used to increase the performance of the DSE.
	 * 
	 * @param systemModelAdapter
	 *            Abstraction of the input models.
	 * @param currentConstraint
	 *            Constraint to be transformed (input).
	 * @return Possibly empty collection of transformed {@link ExplorationConstraint}s.
	 * @throws ExplorationException
	 *             If the transformation fails for any reason.
	 */
	Collection<ExplorationConstraint<?>> transformConstraint(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
			GenericConstraint<?> currentConstraint) throws ExplorationException;
}
