/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service;

import java.util.List;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.exception.ExplorationException;
import org.opt4j.core.Genotype;

/**
 * Service that manages repair functions that can be applied to {@link Genotype}s.
 * 
 * @author diewald
 */
public interface IExplorationRepairService extends IExplorationService<IRepairModule<?>> {

	/**
	 * Returns the collection of {@link IRepairModule} instances that shall be applied to the given
	 * {@link Genotype}.
	 * The {@link IRepairModule}s shall be executed in the given order.
	 * 
	 * @param genotype
	 *            {@link Genotype} that could be repaired.
	 * @param systemModelAdapter
	 *            current {@link SystemModelAdapter} of the DSE.
	 * @return List of applicable {@link IRepairModule}s (in order).
	 * @throws ExplorationException
	 *             if any applicable {@link IRepairModule} could not be instantiated.
	 */
	<T extends Genotype> List<IRepairModule<T>> getRepairFunctionsOf(T genotype,
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) throws ExplorationException;
}
