/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service.internal;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.exception.ExplorationException;
import org.fortiss.af3.exploration.alg.service.IExplorationConstraintTransformationService;
import org.fortiss.af3.exploration.alg.service.IExplorationContraintTransformationModule;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.model.GenericConstraint;
import org.fortiss.tooling.kernel.service.base.EObjectAwareServiceBase;

/**
 * Provides the {@link IExplorationConstraintTransformationService}.
 * 
 * @author diewald
 */
public class ExplorationConstraintTransformationService extends
		EObjectAwareServiceBase<IExplorationContraintTransformationModule<?>> implements
		IExplorationConstraintTransformationService {

	/** The evaluator provider extension point ID. */
	private static final String EXTENSION_POINT_NAME =
			"org.fortiss.af3.exploration.alg.constraintTransformationModule";

	/** The evaluator provider configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "constraintTransformationModule";

	/** The evaluator class attribute name. */
	private static final String HANDLER_CLASS_ATTRIBUTE_NAME = "module";

	/** Holds the instance of the evaluation service. */
	private volatile static ExplorationConstraintTransformationService INSTANCE;

	/** The instance (singleton) of the ExplorationTransformationService. */
	public static synchronized ExplorationConstraintTransformationService getInstance() {
		if(INSTANCE == null) {
			INSTANCE = new ExplorationConstraintTransformationService();
		}
		return INSTANCE;
	}

	/** Constructor. */
	private ExplorationConstraintTransformationService() {
		// Prevent direct instantiation.
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionDescription() {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionLabel() {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws ExplorationException
	 */
	@Override
	public Collection<ExplorationConstraint<?>> getTransformedConstraints(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
			Collection<GenericConstraint<?>> genericConstraints) throws ExplorationException {
		Collection<ExplorationConstraint<?>> transformedConstraints = new ArrayList<>();
		for(GenericConstraint<?> currentConstraint : genericConstraints) {
			for(IExplorationContraintTransformationModule<?> module : getRegisteredHandlers(currentConstraint
					.getClass())) {
				transformedConstraints.addAll(module.transformConstraint(systemModelAdapter,
						currentConstraint));
			}
		}

		return transformedConstraints;
	}

	/** {@inheritDoc} */
	@Override
	protected String getExtensionPointName() {
		return EXTENSION_POINT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getConfigurationElementName() {
		return CONFIGURATION_ELEMENT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getHandlerClassAttribute() {
		return HANDLER_CLASS_ATTRIBUTE_NAME;
	}
}
